﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Collections.ObjectModel;

namespace HeuristicLab.OKB.Cockpit.Admin {
  public static class EntityEditorSupport {

    public static IEnumerable<T> GetObjects<T>(DataTable table, IEnumerable<string> propertyNames) where T : class {
      foreach (DataRow row in table.Rows) {
        if (row["Id"] != DBNull.Value) {
          T o = Activator.CreateInstance<T>();
          foreach (string name in propertyNames)
            typeof(T).GetProperty(name).SetValue(o, row[name] == DBNull.Value ? null : (object)row[name], null);
          yield return o;
        }
      }
    }

    public static void ShowSelector<T>(Func<T, bool> filter, Action<IEnumerable<T>> updater) where T : class {
      var loader = new TableLoadWorker(typeof(T).Name, false);
      loader.RunWorkerCompleted += (s, a) => {
        var objects = GetObjects<T>(loader.Table, new[] { "Id", "Name", "Description", }).ToList();
        RelationEditorWindow relationEditor = new RelationEditorWindow() {
          Title = typeof(T).Name,
          AvailableObjects = new ObservableCollection<object>(objects.Cast<object>()),
          SelectedObjects = new ObservableCollection<object>(objects.Where(o => filter(o)).Cast<object>()),
        };
        if (relationEditor.ShowDialog() == true) {
          updater(relationEditor.SelectedObjects.Cast<T>());
        }
      };
      loader.RunWorkerAsync();
    }

  }
}
